
.equ	RX_DDR			= RX_PORT -1
.equ	RX_PIN			= RX_PORT -2
.equ	TX_DDR			= TX_PORT -1
.equ	TX_PIN			= TX_PORT -2

.if RX_PORT == TX_PORT && RX == TX
.set	Use1Wire		= 1
.else
.set	Use1Wire		= 0
.endif

.equ	UartLoop		= 29 			; don't change, except timing of putc, getc are changed
.equ	BaudTolerance	= 3

.if !UseAutobaud
.set	Baudrate		= XTAL / BootBaudrate - UartLoop
.endif

.set	SUCCESS			= 0x30
.set	ERRORVERIFY		= 0xC0
.set	ERRORCOMMAND	= 0xC1
.set	ERRORCRC		= 0xC2
.set	ERRORBOUNDS		= 0xC3
.set	ERRORCRYPT		= 0xC4
.set	ERRORPROG		= 0xC5
.set	ERRORVERSION	= 0xC6

.if UseCrypt
.set	SUCCESS			= SUCCESS | 0x01
.if UseCryptFLASH
.set	SUCCESS			= SUCCESS | 0x02
.endif
.if UseCryptE2
.set	SUCCESS			= SUCCESS | 0x04
.endif
.endif
.if UseVersioning
.set	SUCCESS			= SUCCESS | 0x08
.endif

.equ	POLYNOM			= 0xA001		; CRC Polynom

.def	feedl			= r4			; feedback pointer
.def	feedh			= r5
.def	zx				= r7			; RAMPZ			
.def	crcl			= r8			; CRC 16Bit
.def	crch			= r9
.def	zerol			= r10			; zero
.def	zeroh			= r11
.def	baudl			= r12			; baudrate
.def	baudh			= r13
.def	sraml			= r14			; SRAM buffer start 
.def	sramh			= r15
.def	paral			= r16			; params for UART
.def	parah			= r17
.def	polyl			= r18			; Polynom CRC
.def	polyh			= r19
.def	cnt				= r20			; counter in UART loops
.def	flag			= r21			; flag for crypto
.def	cmdl			= r22			; commands
.def	cmdh			= r23
.def	cntl			= r24			; temp/baudrate
.def	cnth			= r25

.if FlashEnd > 0x7FFF
.set	XMega			= 1				; ATMega with big FLASH
.else
.set	XMega			= 0
.endif

.ifdef FirstBootStart
.set	BLS				= 1				; Bootloader Section supported
.else
.set	BLS				= 0
.endif

; calculate Bootloader start address
.ifdef FirstBootStart
.set	FirstBoot		= FirstBootStart
.else
.set	FirstBoot		= FlashEnd +1
.endif

.ifdef SecondBootStart
.set	SecondBoot		= SecondBootStart
.else
.set	SecondBoot		= FlashEnd +1
.endif

.ifdef ThirdBootStart
.set	ThirdBoot		= ThirdBootStart
.else
.set	ThirdBoot		= FlashEnd +1
.endif

.ifdef FourthBootStart
.set	FourthBoot		= FourthBootStart
.else
.set	FourthBoot		= FlashEnd +1
.endif

.if BootCodeSize
.if BLS
.set 	FlashSize		= FlashEnd +1
.if FlashSize - FirstBoot >= BootCodeSize >> 1
.equ 	BootStart		= FirstBoot
.message "Please program Boot Fuse to First Boot Start !"
.elif FlashSize - SecondBoot >= BootCodeSize >> 1
.equ 	BootStart		= SecondBoot
.message "Please program Boot Fuse to Second Boot Start !"
.elif FlashSize - ThirdBoot >= BootCodeSize >> 1
.equ 	BootStart		= ThirdBoot
.message "Please program Boot Fuse to Third Boot Start !"
.elif FlashSize - FourthBoot >= BootCodeSize >> 1
.equ 	BootStart		= FourthBoot
.message "Please program Boot Fuse to Fourth Boot Start !"
.else
.equ 	BootStart		= 0
.error "BootStart is undefinied !"
.endif
.else
.set 	FlashSize		= ((FlashEnd +1) * 2 - BootCodeSize) >> 1
.equ 	BootStart		= FlashSize - FlashSize % PageSize
.message "Compile Bootloader without Boot Section !"
.endif
.else
.equ 	BootStart		= 0
.message "Please set used CSEG to BootSize !"
.endif

.set	BootPages		= (FlashEnd +1 - BootStart) / PageSize

.if UseVersioning  
.if BLS
.set	AppVerAddr		= BootStart * 2 - 4		
.else
.set	AppVerAddr		= BootStart * 2 - 6
.endif
.endif

; some redefinition
.ifndef WDCE
.equ	WDCE			= WDDE
.endif

.ifndef	WDTCR
.equ	WDTCR			= WDTCSR
.endif

.ifndef	SPMCSR
.equ	SPMCSR			= SPMCR
.endif

.ifndef	RWWSRE
.ifdef	ASRE
.equ	RWWSRE			= ASRE
.endif
.endif

.ifndef	SPMEN
.equ	SPMEN			= SELFPRGEN
.endif

.ifndef EEMWE
.equ	EEMWE			= EEMPE
.endif

.ifndef EEWE
.equ	EEWE			= EEPE
.endif


; macros
.macro	xout
.if	@0 > 0x3F
	sts	@0, @1
.else
	out	@0, @1
.endif
.endmacro

.macro	xin
.if	@1 > 0x3F
	lds	@0, @1
.else
	in	@0, @1
.endif
.endmacro

.macro	xwdr
.if UseWDR
		wdr
.endif
.endmacro

.macro	xlpm
.if XMega
        elpm    @0, @1
.else
        lpm     @0, @1
.endif
.endmacro

.macro	jmpapp
.if BLS
.if FlashEnd > 0x0FFF
		jmp		0x0000
.else
		rjmp	0x0000								; run application
.endif
.else
		rjmp	BootStart -1						; run user application
.endif
.endmacro

.macro  rx_0
.if UartInvert
		sbic    RX_PIN, RX			
.else
		sbis    RX_PIN, RX			
.endif
.endmacro

.macro  rx_1
.if UartInvert
		sbis    RX_PIN, RX			
.else
		sbic    RX_PIN, RX			
.endif
.endmacro

.macro tx_1
.if (RX != TX) && (RX_PORT == TX_PORT)
.if UartInvert
		ldi		cnth, (1 << RX)
.else
		ldi		cnth, (1 << TX) | (1 << RX)
.endif
.else
.if UartInvert
		clr		cnth
.else
		ldi		cnth, (1 << TX)
.endif
.endif
.endmacro

.macro tx_0
.if (RX != TX) && (RX_PORT == TX_PORT)
.if UartInvert
		ldi		cnth, (1 << TX) | (1 << RX)
.else
		ldi		cnth, (1 << RX)
.endif
.else
.if UartInvert
		ldi		cnth, (1 << TX)
.else
		clr		cnth
.endif
.endif
.endmacro

.macro tx_out
.if Use1Wire
		xout	TX_DDR, cnth						
.else
		xout	TX_PORT, cnth						
.endif
.endmacro
